import pandas as pd
import matplotlib.pyplot as plt


class SlyModel(object):
    def __init__(self, x=None, y=None, text_size=0.2):
        from sklearn.model_selection import train_test_split
        from sklearn.metrics import accuracy_score
        self.accuracy_score = accuracy_score
        self.text_size = text_size
        if x is None and y is None:
            from sklearn import datasets
            self.digits = datasets.load_digits()
            self.x = pd.DataFrame(self.digits.data)
            self.y = self.digits.target
        else:
            self.x = x
            self.y = y

        self.x_train, self.x_test, self.y_train, self.y_test = train_test_split(
            self.x, self.y, test_size=self.text_size, random_state=111)

    def KNC(self,):
        from sklearn.neighbors import KNeighborsClassifier
        best_score = 0
        best_k = -1  # 超参数
        # 网格搜索
        k_range = range(1, len(self.x.keys()))
        acc_scores = []
        for k in k_range:
            KNN = KNeighborsClassifier(n_neighbors=k)
            KNN.fit(self.x_train, self.y_train)
            score = KNN.score(self.x_test, self.y_test)
            acc_scores.append(score)
            if score > best_score:
                best_k = k
                best_score = score
        y_predict = KNN.predict(self.x_test)
        accurate_score = self.accuracy_score(self.y_test, y_predict)
        plt.figure()
        plt.rcParams["figure.figsize"] = (15.0, 8.0)
        plt.xlabel("k")
        plt.ylabel("accuracy")
        plt.plot(k_range, acc_scores, marker="o")
        plt.xticks(range(0, len(self.x.keys())+1,
                        round(len(self.x.keys())/10)+1))
        plt.show()
        return {'predict': pd.Series(list(y_predict)), 'accurate_score': accurate_score, 'best_k': best_k, 'best_score': best_score}

    def LR(self):
        from sklearn.linear_model import LinearRegression
        linear_reg = LinearRegression()
        linear_reg.fit(self.x_train, self.y_train)
        coef = linear_reg.coef_
        intercept = linear_reg.intercept_
        R2 = linear_reg.score(self.x_test, self.y_test)
        predict = linear_reg.predict(self.x_train)
        return{'coef': coef, 'intercept': intercept, 'R2': R2, 'predict': predict}

    def LGR(self):
        from sklearn.linear_model import LogisticRegression
        lr_model = LogisticRegression()
        lr_model.fit(self.x_train, self.y_train)
        coef = lr_model.coef_
        intercept = lr_model.intercept_
        R2 = lr_model.score(self.x_test, self.y_test)
        predict = lr_model.predict(self.x_train)
        return{'coef': coef, 'intercept': intercept, 'R2': R2, 'predict': predict}

    def DTC(self, max_depth=2, min_samples_split=10):
        from sklearn.tree import DecisionTreeClassifier
        dt_clf = DecisionTreeClassifier(max_depth=max_depth,
                                        min_samples_split=min_samples_split
                                        )
        dt_clf.fit(self.x_train, self.y_train)
        score = dt_clf.score(self.x_test, self.y_test)
        return score



class SlyText(object):
    def __init__(self, *text):
        self.text = text
        print(self.text)

    @property
    def translate(self):
        import translators as ts
        return [ts.bing(i, if_use_cn_host=True) for i in self.text]

    @property
    def gender(self):
        import ngender as nd
        return [nd.guess(i) for i in self.text]

    def compare(self, accurate=True):
        from fuzzywuzzy import fuzz
        if accurate == True:
            return fuzz.ratio(self.text[0], self.text[1])/100
        else:
            return fuzz.partial_ratio(self.text[0], self.text[1])/100

    def sentiment(self, language='C'):
        if language == 'E':
            from textblob import TextBlob
            result = {}
            for i in self.text:
                blob = TextBlob(i)
                result[i] = blob.sentiment[0]
            return result
        elif language == 'C':
            from snownlp import SnowNLP
            result = {}
            for i in self.text:
                s = SnowNLP(i)
                result[i] = s.sentiments
            return result
        else:
            print('Please choose a correct languge!')

    def heart(self):
        for name in self.text:
            try:
                print('\n'.join([''.join([(name[(x-y) % len(list(name))]if((x*0.05)**2+(y*0.1)**2-1)**3-(
                    x*0.05)**2*(y*0.1)**3 <= 0 else' ')for x in range(-30, 30)])for y in range(15, -15, -1)]))
            except:
                print('请输入字符串！')

    @property
    def voice(self):
        import pyttsx3
        engine = pyttsx3.init()
        for i in self.text:
            engine.say(i)
            engine.runAndWait()



import pandas as pd
from tqdm import tqdm
from tqdm import trange
import numpy as np
import random
import scipy.optimize as sco
import time
import sys
from .picture import weights_pie
from .GetStock import GetStock
from .GetStock import GetHolidayStock
import os
sys.path.append(os.path.dirname(__file__) + os.sep + '../')
from ..Log.SlyLog import SlyLog


class SlyStock(object):
    def __init__(self, names=None,
                 start_date='2020-12-01',
                 end_date='2020-12-31',
                 frequency="d",
                 holiday=False,
                 holiday_name=None,
                 before=-21, after=21,
                 no_risk_rate=0.45/5200):
        '''
        马科维兹投资组合
        '''
        self.log = SlyLog('Markowit')
        self.log.red(
            'Please make sure that all of the stocks are in the market!')
        self.log.green('initializing...')
        self.no_risk_rate = no_risk_rate
        self.frequency = frequency
        self.names = names
        self.start_date = start_date
        self.end_date = end_date
        if holiday == False:
            self.gs = GetStock(names=names,
                               start_date=start_date,
                               end_date=end_date,
                               frequency=frequency)
            #self.combinations = self.gs.combine
            self.information = self.gs.stock_data
        else:
            if holiday_name == None:
                holiday_name = '国庆节'
                self.holiday_name = holiday_name
            start_date = self.start_date.replace('-', '')
            end_date = self.end_date.replace('-', '')
            self.ghs = GetHolidayStock(names=names,
                                       start_date=start_date,
                                       end_date=end_date,
                                       holiday=holiday_name,
                                       frequency=frequency,
                                       before=before, after=after)
            self.information = self.ghs.HolidayNearbyData
            self.combinations = self.ghs.combine
        self.codes = self.information[0]
        self.stock_pair = self.information[1]
        self.stock_data = self.information[2]
        with open('Markowit.txt', 'a', encoding='utf8') as f:
            f.write('\nヾ(•ω•`)o\n')
            f.write(time.asctime(time.localtime(time.time())))
            f.write('\n')
            f.write(f'time: {self.start_date}--{self.end_date}')
            f.write('\n')
            f.write(f'stocks: {self.names}')
            f.write('\n')
            f.write(f'frequency: {self.frequency}')
            f.write('\n')
            f.write(str(self.stock_pair))
            f.write('\n')
        self.stock_data.to_csv('data.csv', encoding='utf8', index=False)

    def correlation(self, x, y):
        if type(x) == list:
            x = np.array(x)
        if type(y) == list:
            y = np.array(y)
        return (np.dot([xi - np.mean(x) for xi in x], [yi - np.mean(y) for yi in y])/(len(x)-1))/((x.std())*(y.std()))

    def weight(self, lists=['A', 'B', 'C'], mode='dict'):
        if mode == 'dict':
            weights = np.random.dirichlet(np.ones(len(lists)), size=1)[0]
            rand = {}
            for i in range(0, len(lists)):
                rand[lists[i]] = weights[i]
            return rand
        elif mode == 'list':
            weights = np.random.dirichlet(np.ones(len(lists)), size=1)[0]
            rand = {}
            for i in range(0, len(lists)):
                rand[lists[i]] = weights[i]
            return list(rand.values())

    def sharp(self, weights=[], stock_list=[]):
        if weights == []:
            weights = len(self.names) * [1. / len(self.names), ]
        if stock_list == []:
            stock_list = self.names
        df_temp = self.stock_data
        df_temp = df_temp[['pctChg', 'name']]
        df = df_temp[['name']]
        df['pctChg'] = df_temp['pctChg'].astype('float')
        del df_temp
        covs = []
        for i in range(0, len(stock_list)):
            for j in stock_list[i:]:
                if stock_list[i] != j:
                    covs.append([stock_list[i], j])
        for i in covs:
            x = list(df[df['name'] == i[0]]['pctChg'])
            y = list(df[df['name'] == i[1]]['pctChg'])
            i.append(self.correlation(x, y))
        dic = {}
        rate = 0
        risk = 0
        c = 0
        rand = {}
        for i in stock_list:
            weight = weights[c]
            rand[i] = weight
            data = list(df[df['name'] == i]['pctChg'])
            Mean = np.mean(data)
            Std = np.std(data)
            rate += Mean*weight
            risk += (weight*Std)**2
            dic[i] = weight*Std
            c += 1
        for i in covs:
            risk += 2*i[2]*dic[i[0]]*dic[i[1]]
        sharp = (rate-self.no_risk_rate)/risk
        return {'sharp': sharp, 'rate': rate, 'risk': risk, 'weight': rand}

    def max_sharp(self, weights, *args):
        stock_list = args[0]
        with open('Markowit.txt', 'a', encoding='utf8') as f:
            f.write(f'weights: {weights}')
            f.write('\n')
        return -self.sharp(weights=weights, stock_list=stock_list)['sharp']

    def Markowit(self, stock_list=[], accurate=True, number=500):
        if stock_list == []:
            stock_list = self.names
        with open('Markowit.txt', 'a', encoding='utf8') as f:
            f.write(f'\nφ(*￣0￣)\nbuilding a portfolio for {stock_list}')
            f.write('\n')
        if accurate == True:
            opts = sco.minimize(fun=self.max_sharp,
                                x0=len(stock_list) * [1. / len(stock_list), ],
                                method='SLSQP',
                                args=(stock_list,),
                                bounds=tuple((0, 1)
                                             for x in range(len(stock_list))),
                                constraints={'type': 'eq',
                                             'fun': lambda x: np.sum(x) - 1}
                                )
            count = 0
            result = {}
            ans = list(opts['x'])
            for i in stock_list:
                result[i] = ans[count]
                count += 1
            return [result, -opts['fun']]
        else:
            lists = []
            for i in trange(number):
                weights = self.weight(lists=stock_list, mode='list')
                sharp = self.sharp(
                    weights=weights, stock_list=stock_list)
                lists.append(list(sharp.values()))
            df = pd.DataFrame(
                lists, columns=['sharp', 'rate', 'risk', 'weight'])
            df = df.sort_values('sharp', ascending=False)
            return df

    def portfolio(self, accurate=False, number=500):
        try:
            os.makedirs('./Markowit')
        except:
            pass
        if accurate == False:
            #zuhe = self.combinations
            #zuhe = zuhe[zuhe['amount'] > 1]
            #for i, j in tqdm(zuhe.iterrows()):
            file = str(self.names).replace('[', '').replace(']', '')
            self.log.blue(f'calculating {file} ...')
                # with open('Markowit.txt', 'a', encoding='utf-8') as f:
                #    f.write(f'calculating {file} ...')
                #    f.write('\n')
            df = self.Markowit(
                    stock_list=self.names, accurate=False, number=number)
            df.to_csv(f'Markowit\\{file}.csv', index=False, encoding='utf8')
            path = './Markowit'
            filenames = os.listdir(path)
            result = pd.DataFrame()
            for i in filenames:
                df = pd.read_csv(path+'/'+i)
                max_sharp = df.iloc[0]
                result[i.replace('.csv', '')] = max_sharp
            result = result.T
            result = result.sort_values('sharp', ascending=False)
            result.to_csv('Markowit\\result.csv')
            weights = eval(result.iloc[0]['weight'])
            weights_pie(weights)
            return weights  # 最佳投资组合
        else:
            #zuhe = self.combinations
            #zuhe = zuhe[zuhe['amount'] > 1]
            result = []
            #for i, j in (zuhe.iterrows()):
            file = str(self.names).replace('[', '').replace(']', '')
            self.log.blue(f'calculating {file} ...')
            result.append(self.Markowit(stock_list=self.names, accurate=True))
            df = pd.DataFrame(result, columns=['weights', 'sharp'])
            df = df.sort_values('sharp', ascending=False)
            return df

    @property
    def save_result(self):
        df = self.portfolio(accurate=True)
        df.to_csv('Markowit\\result.csv', index=False, encoding='utf8')
        with open('Markowit.txt', 'a', encoding='utf8') as f:
            f.write(
                '\nヾ(≧▽≦*)oヾ(≧▽≦*)oヾ(≧▽≦*)oヾ(≧▽≦*)oヾ(≧▽≦*)oヾ(≧▽≦*)oヾ(≧▽≦*)oヾ(≧▽≦*)oヾ(≧▽≦*)o\n')
        for i, j in df.iterrows():
            self.log.white(f'weights: {j[0]}    sharp: {j[1]}')
            weights_pie(j[0])
        #return df

    def quit(self):
        self.gs.quit()


if __name__ == '__main__':
    ss = SlyStock(names=['贵州茅台', '五粮液', '隆基股份'])
    print(ss.portfolio(accurate=False, number=500))
    #ss.save_result
    ss.quit()



import setuptools

with open("README.md", "r", encoding='utf8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="Suluoya",
    version="1.8.3",
    author="Suluoya",
    author_email="1931960436@qq.com",
    maintainer='Suluoya',
    maintainer_email='1931960436@qq.com',
    description="Suluoya",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleproject",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=['requests',  # 爬虫
                    'pandas',  # 数据分析
                    'beautifulsoup4',  # 爬虫解析
                    # 'translators == 4.7.9',#翻译
                    # 'fuzzywuzzy == 0.18.0',#字符串模拟匹配
                    # 'ngender == 0.1.1',#性别
                    # 'MyQR == 2.3.1',#二维码
                    # 'pyforest == 1.0.3',#自动导入库
                    # 'wget == 3.2',#下载
                    'urllib3',  # 爬虫
                    # 'you-get == 0.4.1488',#下载视频
                    # 'goose3 == 3.1.6',#文章提取
                    # 'pandas_profiling == 2.9.0',#报告
                    # 'flashtext == 2.7',#关键词查找与替换
                    # 'textblob == 0.15.3',#英文情感
                    # 'snownlp == 0.12.3',#中文情感
                    # 'progressbar == 2.5',#进度条
                    'tqdm',  # 进度条
                    # 'pyttsx3',#语音
                    # 'textract',#提取文档信息
                    # 'newspaper3k',#提取新闻
                    'baostock',  # 股票数据
                    'parsel',  # 爬虫解析
                    # 'pretty_errors',#打印错误
                    # 'fake_useragent',#伪装
                    'lunar_python',  # 日期
                    # 'pandasgui',#可视化界面
                    # 'sweetviz',#数据分析
                    'pyecharts',#图
                    ]
)
